function [Irec, errorPSNR]=jpegBasicoFuncion(I, quality)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                                              %%
%% COMPRESION DE VIDEO                                          %%
%% Grado en Ingeniera de Tecnologas de Telecomunicacin       %%
%% Gustavo A. Ruiz Robredo y Juan A. Michell                    %%
%% ruizrg@unican.es, michellj@unican.es                         %%
%%                                                              %%
%% uu   uu  cccccc   UNIVERSIDAD DE CANTABRIA                   %%
%% uu   uu  cccccc   Dpto. INGENIERIA INFORMATICA Y ELECTRONICA %%
%% uu   uu  cc       FACULTAD DE CIENCIAS                       %%
%%  uuuuu   cccccc   Avda. Los Castros s/n                      %%
%%   uuu    cccccc   39005-SANTANDER (SPAIN)                    %%
%%                                                              %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Matrices de cuantificacion
q_y = [16  11  10  16  24  40  51  61; ...
    12  12  14  19  26  58  60  55; ...
    14  13  16  24  40  57  69  56; ...
    14  17  22  29  51  87  80  62; ...
    18  22  37  56  68 109 103  77; ...
    24  35  55  64  81 104 113  92; ...
    49  64  78  87 103 121 120 101; ...
    72  92  95  98 112 100 103  99];

q_c = [ 17 18 24 47 99 99 99 99; ...
    18 21 26 66 99 99 99 99; ...
    24 26 56 99 99 99 99 99; ...
    47 66 99 99 99 99 99 99; ...
    99 99 99 99 99 99 99 99; ...
    99 99 99 99 99 99 99 99; ...
    99 99 99 99 99 99 99 99; ...
    99 99 99 99 99 99 99 99 ];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Conversion RGB -> YcrYcb 4:2:0
RGB=[ 0.299 0.587 0.114; -0.1687 -0.3313 0.5; 0.5 -0.4187 -0.0813];
for x=1:size(I,1)
    for   y=1:size(I,2)
        Ycbcr(x,y,1:3)=RGB*double( [I(x,y,1); I(x,y,2); I(x,y,3)])+ [0; 128; 128];
    end
end

Y0=Ycbcr(:,:,1); %luma
Cb=Ycbcr(:,:,2); % Croma cb
Cr=Ycbcr(:,:,3); % Croma cr

% Definicin filtro 'average' 2x2
h = fspecial('average',[2 2]);

% Filtrado de las cromas
Cbh = imfilter(Cb,h,'replicate'); % Filtrado cr
Crh = imfilter(Cr,h,'replicate'); % Filtrado cb

% Subsampling 4:2:0
Cb0=Cbh(2:2:size(Cbh,1),2:2:size(Cbh,2));
Cr0=Crh(2:2:size(Crh,1),2:2:size(Crh,2));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 2D DCT para bloques 8x8
% Offset -128
Y1 =int16(Y0)-128;
Cb1=int16(Cb0)-128;
Cr1=int16(Cr0)-128;

% 2D DCT
dct=@(block_struct) dct2( block_struct.data);
Y2=blockproc(Y1,   [8 8], dct);
Cb2=blockproc(Cb1, [8 8], dct);
Cr2=blockproc(Cr1, [8 8], dct);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Cuantificacin
% Calculo del factor de escalado
if( quality <= 50)
    scale=50/quality;
else
    scale=2-2*quality/100;
end

% Escalado
q_ys=round(q_y*scale);
q_cs=round(q_c*scale);

% Cuantificacin
Y3=blockproc (Y2,   [8 8], @(block_struct) round(round( block_struct.data ./ q_ys)) );
Cb3=blockproc (Cb2, [8 8], @(block_struct) round(round( block_struct.data ./ q_cs)) );
Cr3=blockproc (Cr2, [8 8], @(block_struct) round(round( block_struct.data ./ q_cs)) );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Cuantificacin inversa: Reescalado
Y4=blockproc (Y3,   [8 8], @(block_struct) round(round( block_struct.data .* q_ys)) );
Cb4=blockproc (Cb3, [8 8], @(block_struct) round(round( block_struct.data .* q_cs)) );
Cr4=blockproc (Cr3, [8 8], @(block_struct) round(round( block_struct.data .* q_cs)) );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 2D IDCT para bloques 8x8
idct=@(block_struct) idct2(block_struct.data);

Y5=blockproc(Y4,   [8 8], idct);
Cb5=blockproc(Cb4, [8 8], idct);
Cr5=blockproc(Cr4, [8 8], idct);

% Offset
Y6=round(round(Y5))+128;
Cb6=round(round(Cb5))+128;
Cr6=round(round(Cr5))+128;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Conversion RGB -> YcrYcb 4:4:4

% Upsampling cromas. Conversion YCbCr 4:2:0 a 4:4:4
Cb6=imresize(Cb6,2);
Cr6=imresize(Cr6,2);

% Conversion RGB -> YcrYcb 4:2:0
RGBinv=inv(RGB);
for x=1:size(Y6,1)
    for   y=1:size(Y6,2)
        Irec(x,y,1:3)=uint8(RGBinv *( [Y6(x,y); Cb6(x,y); Cr6(x,y)] - [0; 128; 128]));
    end
end

% Calculo PSNR
errorPSNR=PSNR(I,Irec);

